@@ -1,12 +1,14 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 'Action'
-import gtk
+import gettext
+
+from gi.repository import GLib, Gtk
+
 from tryton.gui.window.view_form.screen import Screen
 import tryton.rpc as rpc
 import tryton.common as common
 from tryton.pyson import PYSONDecoder
-import gettext
 from tryton.signal_event import SignalEvent
 from tryton.gui.window.win_form import WinForm
 from tryton.common import RPCExecute, RPCException
@@ -57,15 +59,15 @@ class Action(SignalEvent):
         search_context['context'] = self.context
         search_context['_user'] = rpc._USER
         search_value = PYSONDecoder(search_context).decode(
-            self.action['pyson_search_value'] or '{}')
+            self.action['pyson_search_value'] or '[]')
 
-        self.widget = gtk.Frame()
+        self.widget = Gtk.Frame()
         self.widget.set_border_width(0)
 
-        vbox = gtk.VBox(homogeneous=False, spacing=3)
+        vbox = Gtk.VBox(homogeneous=False, spacing=3)
         self.widget.add(vbox)
 
-        self.title = gtk.Label()
+        self.title = Gtk.Label()
         self.widget.set_label_widget(self.title)
         self.widget.set_label_align(0.0, 0.5)
         self.widget.show_all()
@@ -74,7 +76,8 @@ class Action(SignalEvent):
             mode=self.action['view_mode'], context=self.context,
             view_ids=view_ids, domain=self.domain,
             search_value=search_value, row_activate=self.row_activate)
-        vbox.pack_start(self.screen.widget, expand=True, fill=True)
+        vbox.pack_start(
+            self.screen.widget, expand=True, fill=True, padding=0)
         self.screen.signal_connect(self, 'record-message',
             self._active_changed)
 
@@ -100,7 +103,8 @@ class Action(SignalEvent):
                     'id': (self.screen.current_record.id
                         if self.screen.current_record else None),
                     'ids': [r.id for r in self.screen.selected_records],
-                    }, context=self.screen.context.copy(), warning=False)
+                    }, context=self.screen.group._context.copy(),
+                warning=False)
         else:
             def callback(result):
                 if result:
@@ -144,4 +148,4 @@ class Action(SignalEvent):
             def display():
                 if self.screen.widget.props.window:
                     self.display()
-            gtk.idle_add(display)
+            GLib.idle_add(display)
