@@ -174,7 +174,7 @@ class DictIntegerEntry(DictEntry):
 
     def set_value(self, value):
         if value is not None:
-            txt_val = locale.format('%d', value, True)
+            txt_val = locale.format_string('%d', value, True)
         else:
             txt_val = ''
         self.widget.set_text(txt_val)
@@ -211,7 +211,8 @@ class DictFloatEntry(DictIntegerEntry):
             self.widget.digits = None
         self.widget.set_width_chars(self.width)
         if value is not None:
-            txt_val = locale.format('%.*f', (digits[1], value), True)
+            txt_val = locale.localize(
+                '{0:.{1}f}'.format(value, digits[1]), True)
         else:
             txt_val = ''
         self.widget.set_text(txt_val)
