@@ -1,10 +1,20 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
-from .char import Char
+import gettext
 import webbrowser
+from urllib.parse import urljoin, urlencode
+
+from gi.repository import Gtk
+
+from .char import Char
+from .widget import Widget, TranslateMixin
 import tryton.common as common
 from tryton.config import CONFIG
+from tryton.common.common import selection
+from tryton.common.underline import set_underline
+from tryton.rpc import CONNECTION
+
+_ = gettext.gettext
 
 
 class URL(Char):
@@ -14,29 +24,26 @@ class URL(Char):
         super(URL, self).__init__(view, attrs)
 
         self.tooltips = common.Tooltips()
-        self.button = gtk.Button()
-        img = gtk.Image()
-        img.set_from_stock('tryton-web-browser', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.button.set_image(img)
-        self.button.set_relief(gtk.RELIEF_NONE)
+        self.button = Gtk.Button()
+        self.button.set_image(common.IconFactory.get_image(
+                'tryton-public', Gtk.IconSize.SMALL_TOOLBAR))
+        self.button.set_relief(Gtk.ReliefStyle.NONE)
         self.button.connect('clicked', self.button_clicked)
-        self.button.set_alignment(0.5, 0.5)
-        self.widget.pack_start(self.button, expand=False, fill=False)
-        self.widget.set_focus_chain([self.entry])
+        self.widget.pack_start(
+            self.button, expand=False, fill=False, padding=0)
 
-    def display(self, record, field):
-        super(URL, self).display(record, field)
+    def display(self):
+        super(URL, self).display()
         self.set_tooltips()
-        if record and 'icon' in self.attrs:
+        if self.record and 'icon' in self.attrs:
             icon = self.attrs['icon']
-            if icon in record.group.fields:
-                value = record[icon].get_client(record) or 'tryton-web-browser'
+            if icon in self.record.group.fields:
+                value = self.record[icon].get_client(self.record)
+                value = value if value else 'tryton-public'
             else:
                 value = icon
-            common.ICONFACTORY.register_icon(value)
-            img = gtk.Image()
-            img.set_from_stock(value, gtk.ICON_SIZE_SMALL_TOOLBAR)
-            self.button.set_image(img)
+            self.button.set_image(common.IconFactory.get_image(
+                    value, Gtk.IconSize.SMALL_TOOLBAR))
 
     def set_tooltips(self):
         value = self.entry.get_text()
@@ -53,10 +60,6 @@ class URL(Char):
             self.entry.hide()
         else:
             self.entry.show()
-        if value and CONFIG['client.fast_tabbing']:
-            self.widget.set_focus_chain([self.button])
-        else:
-            self.widget.set_focus_chain([self.entry])
         self.button.set_sensitive(True)
 
     def button_clicked(self, widget):
@@ -117,3 +120,49 @@ class SIP(URL):
         else:
             self.tooltips.set_tip(self.button, '')
             self.tooltips.disable()
+
+
+class HTML(Widget, TranslateMixin):
+    "HTML"
+
+    def __init__(self, view, attrs):
+        super().__init__(view, attrs)
+        self.widget = Gtk.HBox()
+        self.mnemonic_widget = self.button = Gtk.LinkButton()
+        self.button.set_label(set_underline(attrs['string']))
+        self.button.set_use_underline(True)
+        self.button.set_alignment(0, 0.5)
+        self.widget.pack_start(
+            self.button, expand=False, fill=False, padding=0)
+
+        if attrs.get('translate'):
+            self.widget.pack_start(
+                self.translate_button(), expand=False, fill=False, padding=0)
+
+    def uri(self, language=None):
+        if not self.record or self.record.id < 0:
+            uri = ''
+        else:
+            path = ['ir/html', self.model_name, str(self.record.id),
+                self.field_name]
+            params = {
+                'language': language or CONFIG['client.lang'],
+                'title': CONFIG['client.title'],
+                }
+            uri = urljoin(
+                CONNECTION.url + '/', '/'.join(path) + '?' + urlencode(params))
+        return uri
+
+    def display(self):
+        super().display()
+        self.button.set_uri(self.uri())
+
+    def _readonly_set(self, value):
+        super()._readonly_set(value)
+        self.button.set_sensitive(not value)
+
+    def translate_dialog(self, languages):
+        languages = {l['name']: l['code'] for l in languages}
+        result = selection(_('Choose a language'), languages)
+        if result:
+            webbrowser.open(self.uri(language=result[1]), new=2)
